@@ -100,7 +100,7 @@ class MiscTests(unittest.TestCase):
         mock_open.return_value = magic
         magic.readline.return_value = _disk_info
         release = misc.get_release()
-        self.assertEqual(release.name, 'Ubuntu-Server')
+        self.assertEqual(release.name, 'Ubuntu Server')
         self.assertEqual(release.version, '10.04.1 LTS')
 
     @mock.patch('builtins.open')
@@ -110,22 +110,22 @@ class MiscTests(unittest.TestCase):
         self.assertEqual(release.name, 'Ubuntu')
         self.assertEqual(release.version, '')
 
-    #@mock.patch('os.path.exists')
-    #def windows_startup_folder(self, mock_exists):
-    #    #mock_exists.return_value = True
-    #    locations = [
-    #        # Windows 7
-    #        'ProgramData/Microsoft/Windows/Start Menu/Programs/Startup',
-    #        # Windows XP
-    #        'Documents and Settings/All Users/Start Menu/Programs/Startup',
-    #        # Windows NT
-    #        'Winnt/Profiles/All Users/Start Menu/Programs/Startup',
-    #                ]
-    #    locations_iter = iter(locations)
-    #    def exists(path):
-    #        return path == locations_iter.next()
+    # @mock.patch('os.path.exists')
+    # def windows_startup_folder(self, mock_exists):
+    #     #mock_exists.return_value = True
+    #     locations = [
+    #         # Windows 7
+    #         'ProgramData/Microsoft/Windows/Start Menu/Programs/Startup',
+    #         # Windows XP
+    #         'Documents and Settings/All Users/Start Menu/Programs/Startup',
+    #         # Windows NT
+    #         'Winnt/Profiles/All Users/Start Menu/Programs/Startup',
+    #                 ]
+    #     locations_iter = iter(locations)
+    #     def exists(path):
+    #         return path == locations_iter.next()
 
-    #    self.assertEqual(misc.windows_startup_folder('/tmp/tmp.XXXXXX'))
+    #     self.assertEqual(misc.windows_startup_folder('/tmp/tmp.XXXXXX'))
 
     @mock.patch('builtins.open')
     def test_mount_info(self, mock_open):
@@ -221,18 +221,18 @@ class MiscTests(unittest.TestCase):
         self.assertEqual(mock_execute.call_count, 0)
         self.assertEqual(mock_set_list.call_count, 0)
 
-#class PartedServerTests(unittest.TestCase):
-#    def setUp(self):
-#        patcher = mock.patch('ubiquity.parted_server.PartedServer')
-#        patcher.start()
-#        # Probably best to patch at the dialog level rather than each method
-#        # call.
-#        # Maybe implement this once we have tests for PartedServer itself,
-#        # with mocks we can reuse.
-#        self.addCleanup(patcher.stop)
+# class PartedServerTests(unittest.TestCase):
+#     def setUp(self):
+#         patcher = mock.patch('ubiquity.parted_server.PartedServer')
+#         patcher.start()
+#         # Probably best to patch at the dialog level rather than each method
+#         # call.
+#         # Maybe implement this once we have tests for PartedServer itself,
+#         # with mocks we can reuse.
+#         self.addCleanup(patcher.stop)
 #
-#    def test_boot_device(self):
-#        print misc.boot_device()
+#     def test_boot_device(self):
+#         print misc.boot_device()
 
 
 class PrivilegeTests(unittest.TestCase):
@@ -249,6 +249,7 @@ class PrivilegeTests(unittest.TestCase):
         os.getuid.return_value = 0
 
         pwd.getpwuid.return_value.pw_name = 'fakegrp'
+        pwd.getpwuid.return_value.pw_gid = '1000'
         gr = mock.Mock()
         gr.gr_mem = ['fakegrp']
         gr.gr_gid = 1234
@@ -263,8 +264,7 @@ class PrivilegeTests(unittest.TestCase):
     @mock.patch('os.setgroups')
     def test_drop_privileges(self, *args):
         with EnvironmentVarGuard() as env:
-            env['SUDO_UID'] = '1000'
-            env['SUDO_GID'] = '1000'
+            env['PKEXEC_UID'] = '1000'
             misc.drop_privileges()
         os.seteuid.assert_called_once_with(1000)
         os.setegid.assert_called_once_with(1000)
@@ -285,8 +285,7 @@ class PrivilegeTests(unittest.TestCase):
     def test_drop_all_privileges(self, *args):
         pwd.getpwuid.return_value.pw_dir = 'fakeusr'
         with EnvironmentVarGuardRestore():
-            os.environ['SUDO_UID'] = '1000'
-            os.environ['SUDO_GID'] = '1000'
+            os.environ['PKEXEC_UID'] = '1000'
             misc.drop_all_privileges()
             os.setreuid.assert_called_once_with(1000, 1000)
             os.setregid.assert_called_once_with(1000, 1000)
