@@ -6,6 +6,7 @@ from PyQt4 import QtGui
 
 from ubiquity.frontend.kde_components.PartitionBar import PartitionsBar
 
+
 if __name__ == "__main__":
     app = QtGui.QApplication(sys.argv)
     QtGui.QApplication.setStyle("Oxygen")
@@ -18,7 +19,8 @@ if __name__ == "__main__":
 
     pb1.addPartition('/dev/sdb1', 57511125504, 'ext4')
     pb1.addPartition('/dev/sdb5', 2500452864, 'linux-swap')
-    pb1.setResizePartition('/dev/sdb1', 230989824, 55143440896, 52143440896, 'distro')
+    pb1.setResizePartition(
+        '/dev/sdb1', 230989824, 55143440896, 52143440896, 'distro')
 
     pb2 = PartitionsBar(wid)
     layout.addWidget(pb2)
