@@ -1,4 +1,3 @@
-import cairo
 from gi.repository import (
     GObject,
     Gdk,
@@ -89,7 +88,7 @@ class StylizedFrame(Gtk.Alignment):
 GObject.type_register(StylizedFrame)
 
 
-class ResizeWidget(Gtk.HPaned):
+class ResizeWidget(Gtk.Paned):
     __gtype_name__ = 'ResizeWidget'
     __gproperties__ = {
         'part_size': (
@@ -114,7 +113,8 @@ class ResizeWidget(Gtk.HPaned):
 
     def __init__(self, part_size=100, min_size=0, max_size=100,
                  existing_part=None, new_part=None):
-        Gtk.HPaned.__init__(self)
+        Gtk.Paned.__init__(self)
+        self.set_orientation(Gtk.Orientation.HORIZONTAL)
         assert min_size <= max_size <= part_size
         assert part_size > 0
         # The size (b) of the existing partition.
@@ -124,16 +124,29 @@ class ResizeWidget(Gtk.HPaned):
         # The max size (b) that the existing partition can be resized to.
         self.max_size = max_size
 
-        # FIXME: Why do we still need these event boxes to get proper bounds
-        # for the linear gradient?
+        # Use test window to figure out highlight color
+        test_window = Gtk.Window()
+        test_label = Gtk.Label()
+        test_window.add(test_label)
+        style = test_label.get_style_context()
+        self.highlight_color = style.get_background_color(
+            Gtk.StateFlags.SELECTED)
+        self.highlight_color.alpha = 0.5
+
         self.existing_part = existing_part or PartitionBox()
-        eb = Gtk.EventBox()
-        eb.add(self.existing_part)
-        self.pack1(eb, resize=False, shrink=False)
+        frame = Gtk.Frame.new()
+        frame.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+        frame.modify_bg(Gtk.StateFlags.NORMAL, self.highlight_color.to_color())
+        frame.add(self.existing_part)
+        self.pack1(frame, resize=False, shrink=False)
+
         self.new_part = new_part or PartitionBox()
-        eb = Gtk.EventBox()
-        eb.add(self.new_part)
-        self.pack2(eb, resize=False, shrink=False)
+        frame = Gtk.Frame.new()
+        frame.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+        frame.modify_bg(Gtk.StateFlags.NORMAL, self.highlight_color.to_color())
+        frame.add(self.new_part)
+        self.pack2(frame, resize=False, shrink=False)
+
         self.show_all()
         # FIXME hideous, but do_realize fails inexplicably.
         self.connect('realize', self.realize)
@@ -152,6 +165,7 @@ class ResizeWidget(Gtk.HPaned):
         self.new_part.set_size_request(pixels, -1)
 
     def do_draw(self, cr):
+        Gtk.Paned.do_draw(self, cr)
         s1 = self.existing_part.get_allocation().width
         s2 = self.new_part.get_allocation().width
         total = s1 + s2
@@ -201,7 +215,7 @@ class DiskBox(Gtk.Box):
 GObject.type_register(DiskBox)
 
 
-class PartitionBox(StylizedFrame):
+class PartitionBox(Gtk.Alignment):
     __gtype_name__ = 'PartitionBox'
     __gproperties__ = {
         'title': (
@@ -243,7 +257,7 @@ class PartitionBox(StylizedFrame):
         # 4 px between the title and the extra heading
         # 5 px between the extra heading and the size
         # 12 px below the bottom-most element
-        StylizedFrame.__init__(self)
+        Gtk.Alignment.__init__(self)
         vbox = Gtk.Box()
         vbox.set_orientation(Gtk.Orientation.VERTICAL)
         self.logo = Gtk.Image.new_from_icon_name(icon_name,
@@ -284,33 +298,6 @@ class PartitionBox(StylizedFrame):
         size = misc.format_size(size)
         self.size.set_markup('<span size="x-large">%s</span>' % size)
 
-    def render_dots(self):
-        # FIXME: Dots are rendered over the frame.
-        s = cairo.ImageSurface(cairo.FORMAT_ARGB32, 2, 2)
-        cr = cairo.Context(s)
-        cr.set_source_rgb(*gtk_to_cairo_color('#b6b0a9'))
-        cr.rectangle(1, 1, 1, 1)
-        cr.fill()
-        pattern = cairo.SurfacePattern(s)
-        return pattern
-
-    def paint_background(self, c):
-        StylizedFrame.paint_background(self, c)
-        a = self.get_allocation()
-        pattern = self.render_dots()
-        pattern.set_extend(cairo.EXTEND_REPEAT)
-        c.set_source(pattern)
-        c.fill_preserve()
-
-        g = cairo.RadialGradient(a.width / 2, a.height / 2, 0, a.width / 2,
-                                 a.height / 2,
-                                 a.width > a.height and a.width or a.height)
-        g.add_color_stop_rgba(0.00, 1, 1, 1, 1.00)
-        g.add_color_stop_rgba(0.25, 1, 1, 1, 0.75)
-        g.add_color_stop_rgba(0.40, 1, 1, 1, 0.00)
-        c.set_source(g)
-        c.fill_preserve()
-
 GObject.type_register(PartitionBox)
 
 
